@@ -227,12 +227,14 @@ on_screenshot_finished (GObject *source,
   CcBackgroundPanelPrivate *priv = panel->priv;
   GError *error;
   GdkRectangle rect;
+  GdkRectangle workarea_rect;
   GtkWidget *widget;
   GdkPixbuf *pixbuf;
   cairo_surface_t *surface;
   cairo_t *cr;
   int width;
   int height;
+  int primary;
 
   error = NULL;
   g_dbus_connection_call_finish (panel->priv->connection,
@@ -266,12 +268,14 @@ on_screenshot_finished (GObject *source,
   g_object_unref (pixbuf);
 
   /* clear the workarea */
-   widget = WID ("background-desktop-drawingarea");
-  gdk_screen_get_monitor_workarea (gtk_widget_get_screen (widget), 0, &rect);
+  widget = WID ("background-desktop-drawingarea");
+  primary = gdk_screen_get_primary_monitor (gtk_widget_get_screen (widget));
+  gdk_screen_get_monitor_geometry (gtk_widget_get_screen (widget), primary, &rect);
+  gdk_screen_get_monitor_workarea (gtk_widget_get_screen (widget), primary, &workarea_rect);
 
   cairo_save (cr);
   cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-  cairo_rectangle (cr, rect.x, rect.y, rect.width, rect.height);
+  cairo_rectangle (cr, workarea_rect.x - rect.x, workarea_rect.y - rect.y, workarea_rect.width, workarea_rect.height);
   cairo_fill (cr);
   cairo_restore (cr);
 
@@ -300,6 +304,9 @@ get_screenshot_async (CcBackgroundPanel *panel,
   const gchar *method_name;
   GVariant *method_params;
 
+  g_debug ("Trying to capture rectangle %dx%d (at %d,%d)",
+           rectangle->width, rectangle->height, rectangle->x, rectangle->y);
+
   path = g_build_filename (g_get_user_cache_dir (), "gnome-control-center", NULL);
   g_mkdir_with_parents (path, 0700);
 
@@ -340,8 +347,10 @@ on_preview_draw (GtkWidget         *widget,
       && panel->priv->screenshot_path == NULL)
     {
       GdkRectangle rect;
+      int primary;
 
-      gdk_screen_get_monitor_geometry (gtk_widget_get_screen (widget), 0, &rect);
+      primary = gdk_screen_get_primary_monitor (gtk_widget_get_screen (widget));
+      gdk_screen_get_monitor_geometry (gtk_widget_get_screen (widget), primary, &rect);
       get_screenshot_async (panel, &rect);
     }
   else
