@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,8 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * Written by: Matthias Clasen <mclasen@redhat.com>
- *             Stef Walter <stefw@gnome.org>
+ * Written by: Stef Walter <stefw@gnome.org>
  */
 
 #include "config.h"
@@ -37,9 +36,9 @@
 
 
 struct _UmRealmManager {
-        UmRealmProviderProxy parent;
+        UmRealmObjectManagerClient parent;
+        UmRealmProvider *provider;
         guint diagnostics_sig;
-        GHashTable *realms;
 };
 
 typedef struct {
@@ -53,7 +52,7 @@ enum {
 
 static gint signals[NUM_SIGNALS] = { 0, };
 
-G_DEFINE_TYPE (UmRealmManager, um_realm_manager, UM_REALM_TYPE_PROVIDER_PROXY);
+G_DEFINE_TYPE (UmRealmManager, um_realm_manager, UM_REALM_TYPE_OBJECT_MANAGER_CLIENT);
 
 GQuark
 um_realm_error_get_quark (void)
@@ -64,172 +63,51 @@ um_realm_error_get_quark (void)
         return quark;
 }
 
-typedef struct {
-        UmRealmManager *manager;
-        GList *realms;
-        gint outstanding;
-} LoadClosure;
-
-static void
-load_closure_free (gpointer data)
+static gboolean
+is_realm_with_kerberos_and_membership (gpointer object)
 {
-        LoadClosure *load = data;
-        g_list_free_full (load->realms, g_object_unref);
-        g_object_unref (load->manager);
-        g_slice_free (LoadClosure, load);
+        GDBusInterface *interface;
+
+        if (!G_IS_DBUS_OBJECT (object))
+                return FALSE;
+
+        interface = g_dbus_object_get_interface (object, "org.freedesktop.realmd.Kerberos");
+        if (interface == NULL)
+                return FALSE;
+        g_object_unref (interface);
+
+        interface = g_dbus_object_get_interface (object, "org.freedesktop.realmd.KerberosMembership");
+        if (interface == NULL)
+                return FALSE;
+        g_object_unref (interface);
+
+        return TRUE;
 }
 
 static void
-on_realm_proxy_created (GObject *source,
-                        GAsyncResult *result,
-                        gpointer user_data)
+on_interface_added (GDBusObjectManager *manager,
+                    GDBusObject *object,
+                    GDBusInterface *interface)
 {
-        GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
-        LoadClosure *load = g_simple_async_result_get_op_res_gpointer (async);
-        UmRealmManager *self = load->manager;
-        UmRealmKerberos *realm;
-        UmRealmKerberos *have;
-        GError *error = NULL;
-        GDBusProxy *proxy;
-        GVariant *info;
+        g_dbus_proxy_set_default_timeout (G_DBUS_PROXY (interface), G_MAXINT);
+}
 
-        realm = um_realm_kerberos_proxy_new_finish (result, &error);
-        if (error == NULL) {
-                proxy = G_DBUS_PROXY (realm);
-                info = g_variant_new ("(os)",
-                                      g_dbus_proxy_get_object_path (proxy),
-                                      g_dbus_proxy_get_interface_name (proxy));
-
-                /* Add it to the manager, unless race */
-                have = g_hash_table_lookup (self->realms, info);
-                if (have == NULL) {
-                        g_dbus_proxy_set_default_timeout (proxy, G_MAXINT);
-                        g_hash_table_insert (self->realms,
-                                             g_variant_ref_sink (info), realm);
-                        g_signal_emit (self, signals[REALM_ADDED], 0, realm);
-
-                } else {
-                        g_object_unref (realm);
-                        g_variant_unref (info);
-                        realm = have;
-                }
-
-                load->realms = g_list_prepend (load->realms, g_object_ref (realm));
-
-        } else {
-                g_simple_async_result_take_error (async, error);
+static void
+on_object_added (GDBusObjectManager *manager,
+                 GDBusObject *object,
+                 gpointer user_data)
+{
+        if (is_realm_with_kerberos_and_membership (object)) {
+                g_debug ("Saw realm: %s", g_dbus_object_get_object_path (object));
+                g_signal_emit (user_data, signals[REALM_ADDED], 0, object);
         }
-
-        if (load->outstanding-- == 1)
-                g_simple_async_result_complete (async);
-
-        g_object_unref (async);
-}
-
-static void
-um_realm_manager_load (UmRealmManager *self,
-                       GVariant *realms,
-                       GCancellable *cancellable,
-                       GAsyncReadyCallback callback,
-                       gpointer user_data)
-{
-        GSimpleAsyncResult *async;
-        GDBusConnection *connection;
-        LoadClosure *load;
-        GVariantIter iter;
-        GVariant *info;
-        UmRealmKerberos *realm;
-        const gchar *path;
-        const gchar *iface;
-
-        g_return_if_fail (realms != NULL);
-
-        async = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
-                                           um_realm_manager_load);
-        load = g_slice_new0 (LoadClosure);
-        load->manager = g_object_ref (self);
-        g_simple_async_result_set_op_res_gpointer (async, load, load_closure_free);
-
-        connection = g_dbus_proxy_get_connection (G_DBUS_PROXY (self));
-
-        g_variant_iter_init (&iter, realms);
-        while (1) {
-                info = g_variant_iter_next_value (&iter);
-                if (info == NULL)
-                        break;
-                realm = g_hash_table_lookup (self->realms, info);
-                if (realm == NULL) {
-                        g_variant_get (info, "(&o&s)", &path, &iface);
-                        if (g_str_equal (iface, "org.freedesktop.realmd.Kerberos")) {
-                                um_realm_kerberos_proxy_new (connection,
-                                                             G_DBUS_PROXY_FLAGS_NONE,
-                                                             "org.freedesktop.realmd",
-                                                             path, cancellable,
-                                                             on_realm_proxy_created,
-                                                             g_object_ref (async));
-                                load->outstanding++;
-                        }
-                } else {
-                        load->realms = g_list_prepend (load->realms, g_object_ref (realm));
-                }
-                g_variant_unref (info);
-        }
-
-        if (load->outstanding == 0)
-                g_simple_async_result_complete_in_idle (async);
-
-        g_object_unref (async);
-}
-
-static GList *
-um_realm_manager_load_finish (UmRealmManager *self,
-                              GAsyncResult *result,
-                              GError **error)
-{
-        GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (result);
-        LoadClosure *load;
-        GList *realms;
-
-        if (g_simple_async_result_propagate_error (async, error))
-                return NULL;
-
-        load = g_simple_async_result_get_op_res_gpointer (async);
-        realms = g_list_reverse (load->realms);
-        load->realms = NULL;
-        return realms;
-}
-
-static guint
-hash_realm_info (gconstpointer value)
-{
-        const gchar *path, *iface;
-        g_variant_get ((GVariant *)value, "(&o&s)", &path, &iface);
-        return g_str_hash (path) ^ g_str_hash (iface);
 }
 
 static void
 um_realm_manager_init (UmRealmManager *self)
 {
-        self->realms = g_hash_table_new_full (hash_realm_info, g_variant_equal,
-                                              (GDestroyNotify)g_variant_unref,
-                                              g_object_unref);
-}
-
-static void
-um_realm_manager_notify (GObject *obj,
-                         GParamSpec *spec)
-{
-        UmRealmManager *self = UM_REALM_MANAGER (obj);
-        GVariant *realms;
-
-        if (G_OBJECT_CLASS (um_realm_manager_parent_class)->notify)
-                G_OBJECT_CLASS (um_realm_manager_parent_class)->notify (obj, spec);
-
-        if (g_str_equal (spec->name, "realms")) {
-                realms = um_realm_provider_get_realms (UM_REALM_PROVIDER (self));
-                if (realms != NULL)
-                        um_realm_manager_load (self, realms, NULL, NULL, NULL);
-        }
+        g_signal_connect (self, "object-added", G_CALLBACK (on_object_added), self);
+        g_signal_connect (self, "interface-added", G_CALLBACK (on_interface_added), self);
 }
 
 static void
@@ -238,8 +116,10 @@ um_realm_manager_dispose (GObject *obj)
         UmRealmManager *self = UM_REALM_MANAGER (obj);
         GDBusConnection *connection;
 
+        g_clear_object (&self->provider);
+
         if (self->diagnostics_sig) {
-                connection = g_dbus_proxy_get_connection (G_DBUS_PROXY (self));
+                connection = g_dbus_object_manager_client_get_connection (G_DBUS_OBJECT_MANAGER_CLIENT (self));
                 if (connection != NULL)
                         g_dbus_connection_signal_unsubscribe (connection, self->diagnostics_sig);
                 self->diagnostics_sig = 0;
@@ -248,29 +128,17 @@ um_realm_manager_dispose (GObject *obj)
         G_OBJECT_CLASS (um_realm_manager_parent_class)->dispose (obj);
 }
 
-static void
-um_realm_manager_finalize (GObject *obj)
-{
-        UmRealmManager *self = UM_REALM_MANAGER (obj);
-
-        g_hash_table_destroy (self->realms);
-
-        G_OBJECT_CLASS (um_realm_manager_parent_class)->finalize (obj);
-}
-
 static void
 um_realm_manager_class_init (UmRealmManagerClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->notify = um_realm_manager_notify;
         object_class->dispose = um_realm_manager_dispose;
-        object_class->finalize = um_realm_manager_finalize;
 
         signals[REALM_ADDED] = g_signal_new ("realm-added", UM_TYPE_REALM_MANAGER,
                                              G_SIGNAL_RUN_FIRST, 0, NULL, NULL,
                                              g_cclosure_marshal_generic,
-                                             G_TYPE_NONE, 1, UM_REALM_TYPE_KERBEROS);
+                                             G_TYPE_NONE, 1, UM_REALM_TYPE_OBJECT);
 }
 
 static void
@@ -324,80 +192,21 @@ version_compare (const char *version,
         return match;
 }
 
-static void
-on_realm_manager_async_init (GObject *source,
-                             GAsyncResult *result,
-                             gpointer user_data)
-{
-        GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
-        UmRealmManager *self;
-        GError *error = NULL;
-        const gchar *version;
-        GDBusProxy *proxy;
-        GObject *object;
-        guint sig;
-
-        object = g_async_initable_new_finish (G_ASYNC_INITABLE (source), result, &error);
-
-        if (error != NULL)
-                g_simple_async_result_take_error (async, error);
-
-        /* This is temporary until the dbus interface stabilizes */
-        if (object) {
-                version = um_realm_provider_get_version (UM_REALM_PROVIDER (object));
-                if (!version_compare (version, 0, 6)) {
-                        /* No need to bother translators with this temporary message */
-                        g_simple_async_result_set_error (async, UM_REALM_ERROR,
-                                                         UM_REALM_ERROR_GENERIC,
-                                                         "Unsupported version of realmd: %s", version);
-                        g_object_unref (object);
-                        object = NULL;
-                }
-        }
-
-        if (object != NULL) {
-                proxy = G_DBUS_PROXY (object);
-                sig = g_dbus_connection_signal_subscribe (g_dbus_proxy_get_connection (proxy),
-                                                          "org.freedesktop.realmd",
-                                                          "org.freedesktop.realmd.Service",
-                                                          "Diagnostics",
-                                                          NULL,
-                                                          NULL,
-                                                          G_DBUS_SIGNAL_FLAGS_NONE,
-                                                          on_realm_diagnostics,
-                                                          NULL,
-                                                          NULL);
-
-                self = UM_REALM_MANAGER (object);
-                self->diagnostics_sig = sig;
-                g_simple_async_result_set_op_res_gpointer (async, self, g_object_unref);
-        }
-
-        g_simple_async_result_complete (async);
-        g_object_unref (async);
-}
-
 void
 um_realm_manager_new (GCancellable *cancellable,
                       GAsyncReadyCallback callback,
                       gpointer user_data)
 {
-        GSimpleAsyncResult *async;
+        g_debug ("Connecting to realmd...");
 
-        async = g_simple_async_result_new (NULL, callback, user_data,
-                                           um_realm_manager_new);
-
-        g_async_initable_new_async (UM_TYPE_REALM_MANAGER,
-                                    G_PRIORITY_DEFAULT, cancellable,
-                                    on_realm_manager_async_init, g_object_ref (async),
-                                    "g-name", "org.freedesktop.realmd",
-                                    "g-bus-type", G_BUS_TYPE_SYSTEM,
-                                    "g-object-path", "/org/freedesktop/realmd",
-                                    "g-interface-name", "org.freedesktop.realmd.Provider",
-                                    "g-default-timeout", G_MAXINT,
+        g_async_initable_new_async (UM_TYPE_REALM_MANAGER, G_PRIORITY_DEFAULT,
+                                    cancellable, callback, user_data,
+                                    "flags", G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_NONE,
+                                    "name", "org.freedesktop.realmd",
+                                    "bus-type", G_BUS_TYPE_SYSTEM,
+                                    "object-path", "/org/freedesktop/realmd",
+                                    "get-proxy-type-func", um_realm_object_manager_client_get_proxy_type,
                                     NULL);
-
-        g_object_unref (async);
 }
 
 UmRealmManager *
@@ -405,23 +214,66 @@ um_realm_manager_new_finish (GAsyncResult *result,
                              GError **error)
 {
         UmRealmManager *self;
-        GSimpleAsyncResult *async;
+        GDBusConnection *connection;
+        GObject *source_object;
+        const gchar *version;
+        GObject *ret;
+        guint sig;
 
-        g_return_val_if_fail (g_simple_async_result_is_valid (result, NULL,
-                              um_realm_manager_new), NULL);
-        g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+        source_object = g_async_result_get_source_object (result);
+        ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), result, error);
+        g_object_unref (source_object);
 
-        async = G_SIMPLE_ASYNC_RESULT (result);
-        if (g_simple_async_result_propagate_error (async, error))
+        if (ret == NULL)
                 return NULL;
 
-        self = g_simple_async_result_get_op_res_gpointer (async);
-        if (self != NULL)
-                g_object_ref (self);
+        self = UM_REALM_MANAGER (ret);
+        connection = g_dbus_object_manager_client_get_connection (G_DBUS_OBJECT_MANAGER_CLIENT (self));
+
+        g_debug ("Connected to realmd, checking version...");
+
+        /*
+         * TODO: Remove this version checking. This is temporary code, so
+         * just use sync here. Shortly we'll be stabilizing the realmd
+         * interfaces.
+         */
+
+        self->provider = um_realm_provider_proxy_new_sync (connection, G_DBUS_PROXY_FLAGS_NONE,
+                                                           "org.freedesktop.realmd",
+                                                           "/org/freedesktop/realmd",
+                                                           NULL, error);
+        if (self->provider == NULL) {
+                g_object_unref (self);
+                return NULL;
+        }
+
+        version = um_realm_provider_get_version (self->provider);
+        if (version == NULL || !version_compare (version, 0, 7)) {
+                /* No need to bother translators with this temporary message */
+                g_set_error (error, UM_REALM_ERROR, UM_REALM_ERROR_GENERIC,
+                             "realmd version should be at least 0.7");
+                g_object_unref (self);
+                return NULL;
+        }
+
+        sig = g_dbus_connection_signal_subscribe (connection,
+                                                  "org.freedesktop.realmd",
+                                                  "org.freedesktop.realmd.Service",
+                                                  "Diagnostics",
+                                                  NULL,
+                                                  NULL,
+                                                  G_DBUS_SIGNAL_FLAGS_NONE,
+                                                  on_realm_diagnostics,
+                                                  NULL,
+                                                  NULL);
+        self->diagnostics_sig = sig;
+
+        g_debug ("Created realm manager");
         return self;
 }
 
 typedef struct {
+        GDBusObjectManager *manager;
         GCancellable *cancellable;
         GList *realms;
 } DiscoverClosure;
@@ -430,29 +282,12 @@ static void
 discover_closure_free (gpointer data)
 {
         DiscoverClosure *discover = data;
+        g_object_unref (discover->manager);
         g_clear_object (&discover->cancellable);
         g_list_free_full (discover->realms, g_object_unref);
         g_slice_free (DiscoverClosure, discover);
 }
 
-static void
-on_manager_load (GObject *source,
-                 GAsyncResult *result,
-                 gpointer user_data)
-{
-        GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
-        DiscoverClosure *discover = g_simple_async_result_get_op_res_gpointer (async);
-        GError *error = NULL;
-
-        discover->realms = um_realm_manager_load_finish (UM_REALM_MANAGER (source),
-                                                         result, &error);
-        if (error != NULL)
-                g_simple_async_result_take_error (async, error);
-        g_simple_async_result_complete (async);
-
-        g_object_unref (async);
-}
-
 static void
 on_provider_discover (GObject *source,
                       GAsyncResult *result,
@@ -460,16 +295,25 @@ on_provider_discover (GObject *source,
 {
         GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
         DiscoverClosure *discover = g_simple_async_result_get_op_res_gpointer (async);
-        UmRealmManager *self = UM_REALM_MANAGER (source);
+        GDBusObject *object;
         GError *error = NULL;
-        GVariant *realms;
+        gchar **realms;
         gint relevance;
+        gint i;
 
-        um_realm_provider_call_discover_finish (UM_REALM_PROVIDER (self),
-                                                &relevance, &realms, result, &error);
+        um_realm_provider_call_discover_finish (UM_REALM_PROVIDER (source), &relevance,
+                                                &realms, result, &error);
         if (error == NULL) {
-                um_realm_manager_load (self, realms, discover->cancellable,
-                                       on_manager_load, g_object_ref (async));
+                for (i = 0; realms[i]; i++) {
+                        object = g_dbus_object_manager_get_object (discover->manager, realms[i]);
+                        if (object == NULL) {
+                                g_warning ("Realm is not in object manager: %s", realms[i]);
+                        } else {
+                                g_debug ("Discovered realm: %s", realms[i]);
+                                discover->realms = g_list_prepend (discover->realms, object);
+                        }
+                }
+                g_strfreev (realms);
 
         } else {
                 g_simple_async_result_take_error (async, error);
@@ -494,15 +338,18 @@ um_realm_manager_discover (UmRealmManager *self,
         g_return_if_fail (input != NULL);
         g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
+        g_debug ("Discovering realms for: %s", input);
+
         res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
                                          um_realm_manager_discover);
         discover = g_slice_new0 (DiscoverClosure);
+        discover->manager = g_object_ref (self);
         discover->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
         g_simple_async_result_set_op_res_gpointer (res, discover, discover_closure_free);
 
 	options = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
 
-        um_realm_provider_call_discover (UM_REALM_PROVIDER (self), input, options, cancellable,
+        um_realm_provider_call_discover (self->provider, input, options, cancellable,
                                          on_provider_discover, g_object_ref (res));
 
         g_object_unref (res);
@@ -533,7 +380,7 @@ um_realm_manager_discover_finish (UmRealmManager *self,
                 return NULL;
         }
 
-        realms = discover->realms;
+        realms = g_list_reverse (discover->realms);
         discover->realms = NULL;
         return realms;
 }
@@ -541,12 +388,75 @@ um_realm_manager_discover_finish (UmRealmManager *self,
 GList *
 um_realm_manager_get_realms (UmRealmManager *self)
 {
+        GList *objects;
+        GList *realms = NULL;
+        GList *l;
+
         g_return_val_if_fail (UM_IS_REALM_MANAGER (self), NULL);
-        return g_hash_table_get_values (self->realms);
+
+        objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (self));
+        for (l = objects; l != NULL; l = g_list_next (l)) {
+                if (is_realm_with_kerberos_and_membership (l->data))
+                        realms = g_list_prepend (realms, g_object_ref (l->data));
+        }
+
+        g_list_free_full (objects, g_object_unref);
+        return realms;
+}
+
+static void
+string_replace (GString *string,
+                const gchar *find,
+                const gchar *replace)
+{
+        const gchar *at;
+        gssize pos;
+
+        at = strstr (string->str, find);
+        if (at != NULL) {
+                pos = at - string->str;
+                g_string_erase (string, pos, strlen (find));
+                g_string_insert (string, pos, replace);
+        }
+}
+
+gchar *
+um_realm_calculate_login (UmRealmCommon *realm,
+                          const gchar *username)
+{
+        GString *string;
+        const gchar *const *formats;
+        gchar *login = NULL;
+
+        formats = um_realm_common_get_login_formats (realm);
+        if (formats[0] != NULL) {
+                string = g_string_new (formats[0]);
+                string_replace (string, "%U", username);
+                string_replace (string, "%D", um_realm_common_get_name (realm));
+                login = g_string_free (string, FALSE);
+        }
+
+        return login;
+
+}
+
+gboolean
+um_realm_is_configured (UmRealmObject *realm)
+{
+        UmRealmCommon *common;
+        const gchar *configured;
+        gboolean is;
+
+        common = um_realm_object_get_common (realm);
+        configured = um_realm_common_get_configured (common);
+        is = configured != NULL && !g_str_equal (configured, "");
+        g_object_unref (common);
+
+        return is;
 }
 
 static const gchar *
-find_supported_credentials (UmRealmKerberos *realm,
+find_supported_credentials (UmRealmKerberosMembership *membership,
                             const gchar *owner)
 {
         const gchar *cred_owner;
@@ -554,7 +464,7 @@ find_supported_credentials (UmRealmKerberos *realm,
         GVariant *supported;
         GVariantIter iter;
 
-        supported = um_realm_kerberos_get_supported_enroll_credentials (realm);
+        supported = um_realm_kerberos_membership_get_supported_join_credentials (membership);
         g_return_val_if_fail (supported != NULL, NULL);
 
         g_variant_iter_init (&iter, supported);
@@ -576,13 +486,16 @@ on_realm_join_complete (GObject *source,
                         gpointer user_data)
 {
 	GSimpleAsyncResult *async = G_SIMPLE_ASYNC_RESULT (user_data);
+
+	g_debug ("Completed Join() method call");
+
 	g_simple_async_result_set_op_res_gpointer (async, g_object_ref (result), g_object_unref);
 	g_simple_async_result_complete_in_idle (async);
 	g_object_unref (async);
 }
 
 static gboolean
-realm_join_as_owner (UmRealmKerberos *realm,
+realm_join_as_owner (UmRealmObject *realm,
                      const gchar *owner,
                      const gchar *login,
                      const gchar *password,
@@ -591,26 +504,35 @@ realm_join_as_owner (UmRealmKerberos *realm,
                      GAsyncReadyCallback callback,
                      gpointer user_data)
 {
+        UmRealmKerberosMembership *membership;
         GSimpleAsyncResult *async;
         GVariant *contents;
         GVariant *options;
         GVariant *creds;
         const gchar *type;
 
-        type = find_supported_credentials (realm, owner);
-        if (type == NULL)
+        membership = um_realm_object_get_kerberos_membership (realm);
+        g_return_val_if_fail (membership != NULL, FALSE);
+
+        type = find_supported_credentials (membership, owner);
+        if (type == NULL) {
+                g_debug ("Couldn't find supported credential type for owner: %s", owner);
+                g_object_unref (membership);
                 return FALSE;
+        }
 
         async = g_simple_async_result_new (G_OBJECT (realm), callback, user_data,
                                            realm_join_as_owner);
 
         if (g_str_equal (type, "ccache")) {
+                g_debug ("Using a kerberos credential cache to join the realm");
                 contents = g_variant_new_from_data (G_VARIANT_TYPE ("ay"),
                                                     g_bytes_get_data (credentials, NULL),
                                                     g_bytes_get_size (credentials),
                                                     TRUE, (GDestroyNotify)g_bytes_unref, credentials);
 
         } else if (g_str_equal (type, "password")) {
+                g_debug ("Using a user/password to join the realm");
                 contents = g_variant_new ("(ss)", login, password);
 
         } else {
@@ -620,16 +542,20 @@ realm_join_as_owner (UmRealmKerberos *realm,
         creds = g_variant_new ("(ssv)", type, owner, contents);
         options = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
 
-        um_realm_kerberos_call_enroll (realm, creds, options,
-                                       cancellable, on_realm_join_complete, g_object_ref (async));
+        g_debug ("Calling the Join() method with %s credentials", owner);
+
+        um_realm_kerberos_membership_call_join (membership, creds, options,
+                                                cancellable, on_realm_join_complete,
+                                                g_object_ref (async));
 
         g_object_unref (async);
+        g_object_unref (membership);
 
         return TRUE;
 }
 
 gboolean
-um_realm_join_as_user (UmRealmKerberos *realm,
+um_realm_join_as_user (UmRealmObject *realm,
                        const gchar *login,
                        const gchar *password,
                        GBytes *credentials,
@@ -637,19 +563,19 @@ um_realm_join_as_user (UmRealmKerberos *realm,
                        GAsyncReadyCallback callback,
                        gpointer user_data)
 {
-        g_return_val_if_fail (UM_REALM_IS_KERBEROS (realm), FALSE);
+        g_return_val_if_fail (UM_REALM_IS_OBJECT (realm), FALSE);
         g_return_val_if_fail (credentials != NULL, FALSE);
         g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
         g_return_val_if_fail (login != NULL, FALSE);
         g_return_val_if_fail (password != NULL, FALSE);
         g_return_val_if_fail (credentials != NULL, FALSE);
 
-        return realm_join_as_owner (realm, "user", login, password, credentials,
-                                    cancellable, callback, user_data);
+        return realm_join_as_owner (realm, "user", login, password,
+                                    credentials, cancellable, callback, user_data);
 }
 
 gboolean
-um_realm_join_as_admin (UmRealmKerberos *realm,
+um_realm_join_as_admin (UmRealmObject *realm,
                         const gchar *login,
                         const gchar *password,
                         GBytes *credentials,
@@ -657,7 +583,7 @@ um_realm_join_as_admin (UmRealmKerberos *realm,
                         GAsyncReadyCallback callback,
                         gpointer user_data)
 {
-        g_return_val_if_fail (UM_REALM_IS_KERBEROS (realm), FALSE);
+        g_return_val_if_fail (UM_REALM_IS_OBJECT (realm), FALSE);
         g_return_val_if_fail (credentials != NULL, FALSE);
         g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
         g_return_val_if_fail (login != NULL, FALSE);
@@ -669,35 +595,44 @@ um_realm_join_as_admin (UmRealmKerberos *realm,
 }
 
 gboolean
-um_realm_join_finish (UmRealmKerberos *self,
+um_realm_join_finish (UmRealmObject *realm,
                       GAsyncResult *result,
                       GError **error)
 {
+        UmRealmKerberosMembership *membership;
         GError *call_error = NULL;
         gchar *dbus_error;
         GAsyncResult *async;
 
-        g_return_val_if_fail (UM_REALM_IS_KERBEROS (self), FALSE);
+        g_return_val_if_fail (UM_REALM_IS_OBJECT (realm), FALSE);
         g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
+        membership = um_realm_object_get_kerberos_membership (realm);
+        g_return_val_if_fail (membership != NULL, FALSE);
+
         async = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
-        um_realm_kerberos_call_enroll_finish (self, async, &call_error);
+        um_realm_kerberos_membership_call_join_finish (membership, async, &call_error);
+        g_object_unref (membership);
+
         if (call_error == NULL)
                 return TRUE;
 
         dbus_error = g_dbus_error_get_remote_error (call_error);
         if (dbus_error == NULL) {
+                g_debug ("Join() failed because of %s", call_error->message);
                 g_propagate_error (error, call_error);
                 return FALSE;
         }
 
         g_dbus_error_strip_remote_error (call_error);
 
-        if (g_str_equal (dbus_error, "org.freedesktop.realmd.Error.AuthFailed")) {
+        if (g_str_equal (dbus_error, "org.freedesktop.realmd.Error.AuthenticationFailed")) {
+                g_debug ("Join() failed because of invalid/insufficient credentials");
                 g_set_error (error, UM_REALM_ERROR, UM_REALM_ERROR_BAD_LOGIN,
                              "%s", call_error->message);
                 g_error_free (call_error);
         } else {
+                g_debug ("Join() failed because of %s", call_error->message);
                 g_propagate_error (error, call_error);
         }
 
@@ -741,10 +676,16 @@ login_perform_kinit (krb5_context k5,
 
         name = g_strdup_printf ("%s@%s", login, realm);
         code = krb5_parse_name (k5, name, &principal);
-        g_free (name);
 
-        if (code != 0)
+        if (code != 0) {
+                g_debug ("Couldn't parse principal name: %s: %s",
+                         name, krb5_get_error_message (k5, code));
+                g_free (name);
                 return code;
+        }
+
+        g_debug ("Using principal name to kinit: %s", name);
+        g_free (name);
 
         if (filename == NULL)
                 code = krb5_cc_default (k5, &ccache);
@@ -753,6 +694,9 @@ login_perform_kinit (krb5_context k5,
 
         if (code != 0) {
                 krb5_free_principal (k5, principal);
+                g_debug ("Couldn't open credential cache: %s: %s",
+                         filename ? filename : "<default>",
+                         krb5_get_error_message (k5, code));
                 return code;
         }
 
@@ -770,8 +714,12 @@ login_perform_kinit (krb5_context k5,
         krb5_cc_close (k5, ccache);
         krb5_free_principal (k5, principal);
 
-        if (code == 0)
+        if (code == 0) {
+                g_debug ("kinit succeeded");
                 krb5_free_cred_contents (k5, &creds);
+        } else {
+                g_debug ("kinit failed: %s", krb5_get_error_message (k5, code));
+        }
 
         return code;
 }
@@ -814,8 +762,10 @@ kinit_thread_func (GSimpleAsyncResult *async,
                         g_file_get_contents (filename, &contents, &length, &error);
                         if (error == NULL) {
                                 login->credentials = g_bytes_new_take (contents, length);
+                                g_debug ("Read in credential cache: %s", filename);
                         } else {
-                                g_warning ("Couldn't read credential cache: %s", error->message);
+                                g_warning ("Couldn't read credential cache: %s: %s",
+                                           filename, error->message);
                                 g_error_free (error);
                         }
                 }
@@ -843,6 +793,7 @@ kinit_thread_func (GSimpleAsyncResult *async,
 
         if (filename) {
                 g_unlink (filename);
+                g_debug ("Deleted credential cache: %s", filename);
                 g_free (filename);
         }
 
@@ -851,8 +802,7 @@ kinit_thread_func (GSimpleAsyncResult *async,
 }
 
 void
-um_realm_login (const gchar *realm,
-                const gchar *domain,
+um_realm_login (UmRealmObject *realm,
                 const gchar *user,
                 const gchar *password,
                 GCancellable *cancellable,
@@ -861,17 +811,21 @@ um_realm_login (const gchar *realm,
 {
         GSimpleAsyncResult *async;
         LoginClosure *login;
+        UmRealmKerberos *kerberos;
 
-        g_return_if_fail (realm != NULL);
+        g_return_if_fail (UM_REALM_IS_OBJECT (realm));
         g_return_if_fail (user != NULL);
         g_return_if_fail (password != NULL);
         g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
+        kerberos = um_realm_object_get_kerberos (realm);
+        g_return_if_fail (kerberos != NULL);
+
         async = g_simple_async_result_new (NULL, callback, user_data,
                                            um_realm_login);
         login = g_slice_new0 (LoginClosure);
-        login->domain = g_strdup (domain ? domain : realm);
-        login->realm = g_strdup (realm);
+        login->domain = g_strdup (um_realm_kerberos_get_domain_name (kerberos));
+        login->realm = g_strdup (um_realm_kerberos_get_realm_name (kerberos));
         login->user = g_strdup (user);
         login->password = g_strdup (password);
         g_simple_async_result_set_op_res_gpointer (async, login, login_closure_free);
@@ -881,6 +835,7 @@ um_realm_login (const gchar *realm,
                                              G_PRIORITY_DEFAULT, cancellable);
 
         g_object_unref (async);
+        g_object_unref (kerberos);
 }
 
 gboolean
